var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../Utils/NumberUtils", "./Utils/InteractionManager", "./Particle", "./Utils/Point", "./Utils/QuadTree", "./Utils/Rectangle", "../Utils/OptionsUtils"], factory);
    }
})(function (require, exports) {
    "use strict";
    var _Particles_engine;
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Particles = void 0;
    const NumberUtils_1 = require("../Utils/NumberUtils");
    const InteractionManager_1 = require("./Utils/InteractionManager");
    const Particle_1 = require("./Particle");
    const Point_1 = require("./Utils/Point");
    const QuadTree_1 = require("./Utils/QuadTree");
    const Rectangle_1 = require("./Utils/Rectangle");
    const OptionsUtils_1 = require("../Utils/OptionsUtils");
    class Particles {
        constructor(engine, container) {
            this.container = container;
            _Particles_engine.set(this, void 0);
            __classPrivateFieldSet(this, _Particles_engine, engine, "f");
            this.nextId = 0;
            this.array = [];
            this.zArray = [];
            this.limit = 0;
            this.needsSort = false;
            this.lastZIndex = 0;
            this.freqs = {
                links: new Map(),
                triangles: new Map(),
            };
            this.interactionManager = new InteractionManager_1.InteractionManager(__classPrivateFieldGet(this, _Particles_engine, "f"), container);
            const canvasSize = this.container.canvas.size;
            this.quadTree = new QuadTree_1.QuadTree(new Rectangle_1.Rectangle(-canvasSize.width / 4, -canvasSize.height / 4, canvasSize.width * 3 / 2, canvasSize.height * 3 / 2), 4);
            this.movers = __classPrivateFieldGet(this, _Particles_engine, "f").plugins.getMovers(container, true);
            this.updaters = __classPrivateFieldGet(this, _Particles_engine, "f").plugins.getUpdaters(container, true);
        }
        get count() {
            return this.array.length;
        }
        addManualParticles() {
            const container = this.container, options = container.actualOptions;
            for (const particle of options.manualParticles) {
                this.addParticle((0, NumberUtils_1.calcPositionFromSize)({
                    size: container.canvas.size,
                    position: particle.position,
                }), particle.options);
            }
        }
        addParticle(position, overrideOptions, group) {
            const container = this.container, options = container.actualOptions, limit = options.particles.number.limit * container.density;
            if (limit > 0) {
                const countToRemove = this.count + 1 - limit;
                if (countToRemove > 0) {
                    this.removeQuantity(countToRemove);
                }
            }
            return this.pushParticle(position, overrideOptions, group);
        }
        addSplitParticle(parent, splitParticlesOptions) {
            const splitOptions = parent.options.destroy.split, options = (0, OptionsUtils_1.loadParticlesOptions)(__classPrivateFieldGet(this, _Particles_engine, "f"), this.container, parent.options), factor = (0, NumberUtils_1.getValue)(splitOptions.factor);
            options.color.load({
                value: {
                    hsl: parent.getFillColor(),
                },
            });
            if (typeof options.size.value === "number") {
                options.size.value /= factor;
            }
            else {
                options.size.value.min /= factor;
                options.size.value.max /= factor;
            }
            options.load(splitParticlesOptions);
            const offset = splitOptions.sizeOffset ? (0, NumberUtils_1.setRangeValue)(-parent.size.value, parent.size.value) : 0, position = {
                x: parent.position.x + (0, NumberUtils_1.randomInRange)(offset),
                y: parent.position.y + (0, NumberUtils_1.randomInRange)(offset),
            };
            return this.pushParticle(position, options, parent.group, (particle) => {
                if (particle.size.value < 0.5) {
                    return false;
                }
                particle.velocity.length = (0, NumberUtils_1.randomInRange)((0, NumberUtils_1.setRangeValue)(parent.velocity.length, particle.velocity.length));
                particle.splitCount = parent.splitCount + 1;
                particle.unbreakable = true;
                setTimeout(() => {
                    particle.unbreakable = false;
                }, 500);
                return true;
            });
        }
        clear() {
            this.array = [];
            this.zArray = [];
        }
        destroy() {
            this.array = [];
            this.zArray = [];
            this.movers = [];
            this.updaters = [];
        }
        async draw(delta) {
            const container = this.container, canvasSize = this.container.canvas.size;
            this.quadTree = new QuadTree_1.QuadTree(new Rectangle_1.Rectangle(-canvasSize.width / 4, -canvasSize.height / 4, canvasSize.width * 3 / 2, canvasSize.height * 3 / 2), 4);
            container.canvas.clear();
            await this.update(delta);
            if (this.needsSort) {
                this.zArray.sort((a, b) => b.position.z - a.position.z || a.id - b.id);
                this.lastZIndex = this.zArray[this.zArray.length - 1].position.z;
                this.needsSort = false;
            }
            for (const [, plugin] of container.plugins) {
                container.canvas.drawPlugin(plugin, delta);
            }
            for (const p of this.zArray) {
                p.draw(delta);
            }
        }
        getLinkFrequency(p1, p2) {
            const range = (0, NumberUtils_1.setRangeValue)(p1.id, p2.id), key = `${(0, NumberUtils_1.getRangeMin)(range)}_${(0, NumberUtils_1.getRangeMax)(range)}`;
            let res = this.freqs.links.get(key);
            if (res === undefined) {
                res = Math.random();
                this.freqs.links.set(key, res);
            }
            return res;
        }
        getTriangleFrequency(p1, p2, p3) {
            let [id1, id2, id3] = [p1.id, p2.id, p3.id];
            if (id1 > id2) {
                [id2, id1] = [id1, id2];
            }
            if (id2 > id3) {
                [id3, id2] = [id2, id3];
            }
            if (id1 > id3) {
                [id3, id1] = [id1, id3];
            }
            const key = `${id1}_${id2}_${id3}`;
            let res = this.freqs.triangles.get(key);
            if (res === undefined) {
                res = Math.random();
                this.freqs.triangles.set(key, res);
            }
            return res;
        }
        handleClickMode(mode) {
            this.interactionManager.handleClickMode(mode);
        }
        init() {
            var _a;
            const container = this.container, options = container.actualOptions;
            this.lastZIndex = 0;
            this.needsSort = false;
            this.freqs.links = new Map();
            this.freqs.triangles = new Map();
            let handled = false;
            this.updaters = __classPrivateFieldGet(this, _Particles_engine, "f").plugins.getUpdaters(container, true);
            this.interactionManager.init();
            for (const [, plugin] of container.plugins) {
                if (plugin.particlesInitialization !== undefined) {
                    handled = plugin.particlesInitialization();
                }
                if (handled) {
                    break;
                }
            }
            this.addManualParticles();
            if (!handled) {
                for (const group in options.particles.groups) {
                    const groupOptions = options.particles.groups[group];
                    for (let i = this.count, j = 0; j < ((_a = groupOptions.number) === null || _a === void 0 ? void 0 : _a.value) && i < options.particles.number.value; i++, j++) {
                        this.addParticle(undefined, groupOptions, group);
                    }
                }
                for (let i = this.count; i < options.particles.number.value; i++) {
                    this.addParticle();
                }
            }
            container.pathGenerator.init(container);
        }
        push(nb, mouse, overrideOptions, group) {
            this.pushing = true;
            for (let i = 0; i < nb; i++) {
                this.addParticle(mouse === null || mouse === void 0 ? void 0 : mouse.position, overrideOptions, group);
            }
            this.pushing = false;
        }
        async redraw() {
            this.clear();
            this.init();
            await this.draw({ value: 0, factor: 0 });
        }
        remove(particle, group, override) {
            this.removeAt(this.array.indexOf(particle), undefined, group, override);
        }
        removeAt(index, quantity = 1, group, override) {
            if (!(index >= 0 && index <= this.count)) {
                return;
            }
            let deleted = 0;
            for (let i = index; deleted < quantity && i < this.count; i++) {
                const particle = this.array[i];
                if (!particle || particle.group !== group) {
                    continue;
                }
                particle.destroy(override);
                this.array.splice(i--, 1);
                const zIdx = this.zArray.indexOf(particle);
                this.zArray.splice(zIdx, 1);
                deleted++;
                __classPrivateFieldGet(this, _Particles_engine, "f").dispatchEvent("particleRemoved", {
                    container: this.container,
                    data: {
                        particle,
                    },
                });
            }
        }
        removeQuantity(quantity, group) {
            this.removeAt(0, quantity, group);
        }
        setDensity() {
            const options = this.container.actualOptions;
            for (const group in options.particles.groups) {
                this.applyDensity(options.particles.groups[group], 0, group);
            }
            this.applyDensity(options.particles, options.manualParticles.length);
        }
        async update(delta) {
            const container = this.container, particlesToDelete = [];
            container.pathGenerator.update();
            for (const [, plugin] of container.plugins) {
                if (plugin.update) {
                    plugin.update(delta);
                }
            }
            for (const particle of this.array) {
                const resizeFactor = container.canvas.resizeFactor;
                if (resizeFactor && !particle.ignoresResizeRatio) {
                    particle.position.x *= resizeFactor.width;
                    particle.position.y *= resizeFactor.height;
                }
                particle.ignoresResizeRatio = false;
                await this.interactionManager.reset(particle);
                for (const [, plugin] of this.container.plugins) {
                    if (particle.destroyed) {
                        break;
                    }
                    if (plugin.particleUpdate) {
                        plugin.particleUpdate(particle, delta);
                    }
                }
                for (const mover of this.movers) {
                    if (mover.isEnabled(particle)) {
                        mover.move(particle, delta);
                    }
                }
                if (particle.destroyed) {
                    particlesToDelete.push(particle);
                    continue;
                }
                this.quadTree.insert(new Point_1.Point(particle.getPosition(), particle));
            }
            for (const particle of particlesToDelete) {
                this.remove(particle);
            }
            await this.interactionManager.externalInteract(delta);
            for (const particle of container.particles.array) {
                for (const updater of this.updaters) {
                    updater.update(particle, delta);
                }
                if (!particle.destroyed && !particle.spawning) {
                    await this.interactionManager.particlesInteract(particle, delta);
                }
            }
            delete container.canvas.resizeFactor;
        }
        applyDensity(options, manualCount, group) {
            var _a;
            if (!((_a = options.number.density) === null || _a === void 0 ? void 0 : _a.enable)) {
                return;
            }
            const numberOptions = options.number, densityFactor = this.initDensityFactor(numberOptions.density), optParticlesNumber = numberOptions.value, optParticlesLimit = numberOptions.limit > 0 ? numberOptions.limit : optParticlesNumber, particlesNumber = Math.min(optParticlesNumber, optParticlesLimit) * densityFactor + manualCount, particlesCount = Math.min(this.count, this.array.filter((t) => t.group === group).length);
            this.limit = numberOptions.limit * densityFactor;
            if (particlesCount < particlesNumber) {
                this.push(Math.abs(particlesNumber - particlesCount), undefined, options, group);
            }
            else if (particlesCount > particlesNumber) {
                this.removeQuantity(particlesCount - particlesNumber, group);
            }
        }
        initDensityFactor(densityOptions) {
            const container = this.container;
            if (!container.canvas.element || !densityOptions.enable) {
                return 1;
            }
            const canvas = container.canvas.element, pxRatio = container.retina.pixelRatio;
            return canvas.width * canvas.height / (densityOptions.factor * pxRatio ** 2 * densityOptions.area);
        }
        pushParticle(position, overrideOptions, group, initializer) {
            try {
                const particle = new Particle_1.Particle(__classPrivateFieldGet(this, _Particles_engine, "f"), this.nextId, this.container, position, overrideOptions, group);
                let canAdd = true;
                if (initializer) {
                    canAdd = initializer(particle);
                }
                if (!canAdd) {
                    return;
                }
                this.array.push(particle);
                this.zArray.push(particle);
                this.nextId++;
                __classPrivateFieldGet(this, _Particles_engine, "f").dispatchEvent("particleAdded", {
                    container: this.container,
                    data: {
                        particle,
                    },
                });
                return particle;
            }
            catch (e) {
                console.warn(`error adding particle: ${e}`);
                return;
            }
        }
    }
    exports.Particles = Particles;
    _Particles_engine = new WeakMap();
});
